import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { CommonCmsRepository } from "./common-cms.repository";
import { UpdateCommonCmsDto } from "./dto/update-common-cms.dto";
import { CommonCmsEntity } from "./entities/common-cms.entity";
import { NewsletterBannerCmsRepository } from "../newsletter-banner-cms/newsletter-banner-cms.repository";
import { ReeferFriendCmsRepository } from "../reefer-friend-cms/reefer-friend-cms.repository";
import { ReeferFriendRepository } from "src/modules/reefer-friend/reefer-friend.repository";
import { getGraphQLClientWithApiKey } from "src/helper/graphql-client-helper";
import { fetchAllRetailers } from "src/helper/dutchie-plus-query";
import { PageTitleCmsRepository } from "../page-title-cms/page-title-cms.repository";
import {
  RetailerServiceRepository
} from "src/modules/dutchie-plus/custom/retailer-service/retailer-service.repository";
import { PageMetaCmsRepository } from "../page-meta-cms/page-meta-cms.repository";
import {
  AboutUsPageComponentUiRepository
} from "src/modules/component-ui-cms/about-us-page-component-ui/about-us-page-component-ui.repository";
import {
  BlogDetailsPageComponentUiRepository
} from "src/modules/component-ui-cms/blog-details-page-component-ui/blog-details-page-component-ui.repository";
import {
  BlogPageComponentUiRepository
} from "src/modules/component-ui-cms/blog-page-component-ui/blog-page-component-ui.repository";
import {
  BrandPageComponentUiRepository
} from "src/modules/component-ui-cms/brand-page-component-ui/brand-page-component-ui.repository";
import {
  CareerDepartmentPageComponentUiRepository
} from "src/modules/component-ui-cms/career-department-page-component-ui/career-department-page-component-ui.repository";
import {
  CareerDetailsPageComponentUiRepository
} from "src/modules/component-ui-cms/career-details-page-component-ui/career-details-page-component-ui.repository";
import {
  CareerPageComponentUiRepository
} from "src/modules/component-ui-cms/career-page-component-ui/career-page-component-ui.repository";
import {
  CategoriesPageComponentUiRepository
} from "src/modules/component-ui-cms/categories-page-component-ui/categories-page-component-ui.repository";
import {
  CommonComponentUiRepository
} from "src/modules/component-ui-cms/common-component-ui/common-component-ui.repository";
import {
  ContactUsPageComponentUiRepository
} from "src/modules/component-ui-cms/contact-us-page-component-ui/contact-us-page-component-ui.repository";
import {
  FaqPageComponentUiRepository
} from "src/modules/component-ui-cms/faq-page-component-ui/faq-page-component-ui.repository";
import {
  HomePageComponentUiRepository
} from "src/modules/component-ui-cms/home-page-component-ui/home-page-component-ui.repository";
import {
  LocationPageComponentUiRepository
} from "src/modules/component-ui-cms/location-page-component-ui/location-page-component-ui.repository";
import {
  PrivacyPolicyPageComponentUiRepository
} from "src/modules/component-ui-cms/privacy-policy-page-component-ui/privacy-policy-page-component-ui.repository";
import {
  ShopPageComponentUiRepository
} from "src/modules/component-ui-cms/shop-page-component-ui/shop-page-component-ui.repository";
import {
  SpecialPageComponentUiRepository
} from "src/modules/component-ui-cms/special-page-component-ui/special-page-component-ui.repository";
import {
  TermsAndConditionsPageComponentUiRepository
} from "src/modules/component-ui-cms/terms-and-conditions-page-component-ui/terms-and-conditions-page-component-ui.repository";
import { InstagramServiceRepository } from "../../instagram-service/instagram-service.repository";
import { FooterImageRepository } from "src/modules/footer-image/footer-image.repository";
import {
  NewsletterComponentUiRepository
} from "../../component-ui-cms/newsletter-component-ui/newsletter-component-ui.repository";

@Injectable()
export class CommonCmsService {
  constructor(
    @InjectRepository(CommonCmsRepository)
    private readonly repository: CommonCmsRepository,
    @InjectRepository(NewsletterBannerCmsRepository)
    private readonly newsletterBannerCmsRepository: NewsletterBannerCmsRepository,
    @InjectRepository(ReeferFriendCmsRepository)
    private readonly reeferFriendCmsRepository: ReeferFriendCmsRepository,
    @InjectRepository(ReeferFriendRepository)
    private readonly reeferFriendRepository: ReeferFriendRepository,
    @InjectRepository(PageTitleCmsRepository)
    private readonly pageTitleCmsRepository: PageTitleCmsRepository,
    @InjectRepository(RetailerServiceRepository)
    private readonly retailerServiceRepository: RetailerServiceRepository,
    @InjectRepository(PageMetaCmsRepository)
    private readonly pageMetaCmsRepository: PageMetaCmsRepository,
    //
    @InjectRepository(LocationPageComponentUiRepository)
    private readonly locationPageComponentUiRepository: LocationPageComponentUiRepository,
    @InjectRepository(HomePageComponentUiRepository)
    private readonly homePageComponentUiRepository: HomePageComponentUiRepository,
    @InjectRepository(ShopPageComponentUiRepository)
    private readonly shopPageComponentUiRepository: ShopPageComponentUiRepository,
    @InjectRepository(AboutUsPageComponentUiRepository)
    private readonly aboutUsPageComponentUiRepository: AboutUsPageComponentUiRepository,
    @InjectRepository(CategoriesPageComponentUiRepository)
    private readonly categoriesPageComponentUiRepository: CategoriesPageComponentUiRepository,
    @InjectRepository(BrandPageComponentUiRepository)
    private readonly brandPageComponentUiRepository: BrandPageComponentUiRepository,
    @InjectRepository(SpecialPageComponentUiRepository)
    private readonly specialPageComponentUiRepository: SpecialPageComponentUiRepository,
    @InjectRepository(ContactUsPageComponentUiRepository)
    private readonly contactUsPageComponentUiRepository: ContactUsPageComponentUiRepository,
    @InjectRepository(FaqPageComponentUiRepository)
    private readonly faqPageComponentUiRepository: FaqPageComponentUiRepository,
    @InjectRepository(CareerPageComponentUiRepository)
    private readonly careerPageComponentUiRepository: CareerPageComponentUiRepository,
    @InjectRepository(CareerDepartmentPageComponentUiRepository)
    private readonly careerDepartmentPageComponentUiRepository: CareerDepartmentPageComponentUiRepository,
    @InjectRepository(CareerDetailsPageComponentUiRepository)
    private readonly careerDetailsPageComponentUiRepository: CareerDetailsPageComponentUiRepository,
    @InjectRepository(BlogPageComponentUiRepository)
    private readonly blogPageComponentUiRepository: BlogPageComponentUiRepository,
    @InjectRepository(PrivacyPolicyPageComponentUiRepository)
    private readonly privacyPolicyPageComponentUiRepository: PrivacyPolicyPageComponentUiRepository,
    @InjectRepository(TermsAndConditionsPageComponentUiRepository)
    private readonly termsAndConditionsPageComponentUiRepository: TermsAndConditionsPageComponentUiRepository,
    @InjectRepository(BlogDetailsPageComponentUiRepository)
    private readonly blogDetailsPageComponentUiRepository: BlogDetailsPageComponentUiRepository,
    @InjectRepository(CommonComponentUiRepository)
    private readonly commonComponentUiRepository: CommonComponentUiRepository,
    @InjectRepository(InstagramServiceRepository)
    private readonly instagramServiceRepo: InstagramServiceRepository,
    @InjectRepository(FooterImageRepository)
    private readonly footerImageRepo: FooterImageRepository,
    @InjectRepository(NewsletterComponentUiRepository)
    private readonly  newsletterComponentUiRepo: NewsletterComponentUiRepository
  ) {
  }

  async updateOrCreateData(
    dto: UpdateCommonCmsDto
  ): Promise<CommonCmsEntity | Error> {
    const isExists = await this.repository.find({ isDeleted: false });
    if (Object.keys(isExists).length === 0) {
      const data = await this.repository.createEntity(dto);
      if (!data) {
        throw new HttpException(
          Constants.NOT_FOUND,
          HttpStatus.NOT_FOUND
        );
      }
      return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
    } else {
      let id = isExists[0].id;
      const data = await this.repository.updateEntity(dto, id);
      if (!data) {
        throw new HttpException(
          Constants.UPDATE_FAILED,
          HttpStatus.BAD_REQUEST
        );
      }
      return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
    }
  }

  async findAll(): Promise<CommonCmsEntity[] | Error> {
    const commonCmsData = await this.repository.findAll();
    const newsletterCmsData = await this.newsletterBannerCmsRepository.findAll();
    const reeferFriendCMS = await this.reeferFriendCmsRepository.findAll();
    const reeferFriend = await this.reeferFriendRepository.findByFilterQuery({});
    const allRetailers = await this.allRetailers();
    const pageTitles = await this.pageTitleCmsRepository.findAll();
    const retailerServiceData = await this.allRetailersForSideBar();
    const pageMetaCmsData = await this.pageMetaCmsRepository.findAll();
    //
    const locationPageData = await this.locationPageComponentUiRepository.findAll();
    const homePageData = await this.homePageComponentUiRepository.findAll();
    const shopPageData = await this.shopPageComponentUiRepository.findAll();
    const aboutUsPageData = await this.aboutUsPageComponentUiRepository.findAll();
    const categoriesPageData = await this.categoriesPageComponentUiRepository.findAll();
    const brandPageData = await this.brandPageComponentUiRepository.findAll();
    const specialPageData = await this.specialPageComponentUiRepository.findAll();
    const contactUsPageData = await this.contactUsPageComponentUiRepository.findAll();
    const faqPageData = await this.faqPageComponentUiRepository.findAll();
    const careerPageData = await this.careerPageComponentUiRepository.findAll();
    const careerDepartmentPageData = await this.careerDepartmentPageComponentUiRepository.findAll();
    const careerDetailsPageData = await this.careerDetailsPageComponentUiRepository.findAll();
    const blogPageData = await this.blogPageComponentUiRepository.findAll();
    const blogDetailsPageData = await this.blogDetailsPageComponentUiRepository.findAll();
    const privacyPolicyPageData = await this.privacyPolicyPageComponentUiRepository.findAll();
    const termsAndConditionPageData = await this.termsAndConditionsPageComponentUiRepository.findAll();
    const commonPageData = await this.commonComponentUiRepository.findAll();
    const instagramService = await this.instagramServiceRepo.findByFilterQuery({});
    const footerImage = await this.footerImageRepo.findByFilterQuery({});
    const newsletterComponentUi = await this.newsletterComponentUiRepo.findAll();


    const data = {
      commonCMS: commonCmsData,
      newsletterCMS: newsletterCmsData,
      reeferFriendCMS: reeferFriendCMS,
      reeferFriend: reeferFriend,
      allRetailers: allRetailers,
      pageTitles: pageTitles,
      retailerServiceData: retailerServiceData,
      pageMetaCmsData: pageMetaCmsData,
      instagramService: instagramService,
      componentUI: {
        locationPage: locationPageData,
        homePage: homePageData,
        shopPage: shopPageData,
        aboutUsPage: aboutUsPageData,
        categoriesPage: categoriesPageData,
        brandPage: brandPageData,
        specialPage: specialPageData,
        contactUsPage: contactUsPageData,
        faqPage: faqPageData,
        careerPage: careerPageData,
        careerDepartmentPage: careerDepartmentPageData,
        careerDetailsPage: careerDetailsPageData,
        blogPage: blogPageData,
        privacyPolicyPage: privacyPolicyPageData,
        termsAndConditionPage: termsAndConditionPageData,
        blogDetailsPage: blogDetailsPageData,
        commonPage: commonPageData,
        newsletter: newsletterComponentUi
      },
      footerImage: footerImage
    };

    return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
  }

  async fetchRetailers(apiKeyName: string) {
    const graphQLClient = getGraphQLClientWithApiKey(apiKeyName);
    const query = fetchAllRetailers();
    const data = await graphQLClient.request(query);
    return data.retailers.map(retailer => ({ ...retailer, apiKeyName }));
  }

  async allRetailers(): Promise<any | Error> {
    try {
      const promises = [
        this.fetchRetailers("DUTCHIE_PLUS_API_TOKEN_1"),
        this.fetchRetailers("DUTCHIE_PLUS_API_TOKEN_2")
      ];
      const [retailers1, retailers2] = await Promise.all(promises);
      const mergedRetailers = [...retailers1, ...retailers2];

      let retailerList = [];
      const retailers = mergedRetailers;
      for (let i = 0; i < retailers.length; i++) {
        retailerList.push(retailers[i]);
      }
      return retailerList;
    } catch (error) {
      console.log(error);
      return error;
    }
  }

  async allRetailersForSideBar(): Promise<any | Error> {
    try {
      const promises = [
        this.fetchRetailers("DUTCHIE_PLUS_API_TOKEN_1"),
        this.fetchRetailers("DUTCHIE_PLUS_API_TOKEN_2")
      ];
      const [retailers1, retailers2] = await Promise.all(promises);
      const mergedRetailers = [...retailers1, ...retailers2];

      let retailerList = [];
      const retailers = mergedRetailers;
      for (let i = 0; i < retailers.length; i++) {
        const retailerService = await this.retailerServiceRepository.findOne({
          where: {
            retailerId: retailers[i].id,
            isDeleted: false
          }
        });
        retailers[i].retailerMobileNumber = retailerService ? retailerService.retailerMobileNumber : null;
        retailerList.push(retailers[i]);
      }
      return retailerList;
    } catch (e) {
      console.log(e);
      return e;
    }
  }
}