import {
  Body,
  Controller,
  Get,
  HttpCode,
  HttpStatus,
  Post,
  UploadedFiles,
  UseGuards,
  UseInterceptors
} from "@nestjs/common";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { Constants } from "src/utils/constants";
import { FooterCmsService } from "./footer-cms.service";
import { UpdateFooterCmsDto } from "./dto/update-footer-cms.dto";
import { FileFieldsInterceptor } from "@nestjs/platform-express";
import { UploadedMulterFileI } from "../../SpacesModule/SpacesService";

@Controller({ path: "footer-cms", version: Constants.API_VERSION_1 })

export class FooterCmsController {
  constructor(private readonly service: FooterCmsService) {
  }

  @Post()
  @HttpCode(HttpStatus.OK)
  @UseGuards(AdminAuthGuard)
  @UseInterceptors(FileFieldsInterceptor(
    [
      { name: "logo", maxCount: 1 }
    ]
  ))
  updateOrCreateData(
    @Body() dto: UpdateFooterCmsDto,
    @UploadedFiles() files: {
      logo: UploadedMulterFileI
    }
  ) {
    return this.service.updateOrCreateData(dto, files);
  }

  @Get()
  // @UseGuards(AdminAuthGuard)
  findAll() {
    return this.service.findAll();
  }
}
