import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { HeroSectionLeftCmsDefaultRepository } from "./hero-section-left-cms-default.repository";
import { UpdateHeroSectionLeftCmsDefaultDto } from "./dto/update-hero-section-left-cms-default.dto";
import { HeroSectionLeftCmsDefaultEntity } from "./entities/hero-section-left-cms-default.entity";
import { DoSpacesService } from "src/modules/SpacesModule/SpacesService/doSpacesService";

@Injectable()
export class HeroSectionLeftCmsDefaultService {
  constructor(
    @InjectRepository(HeroSectionLeftCmsDefaultRepository)
    private readonly repository: HeroSectionLeftCmsDefaultRepository,
    private readonly doSpaceService: DoSpacesService
  ) {
  }

  async updateOrCreateData(
    dto: UpdateHeroSectionLeftCmsDefaultDto
  ): Promise<HeroSectionLeftCmsDefaultEntity | Error> {
    const isExists = await this.repository.findAll();
    if (Object.keys(isExists).length === 0) {
      const data = await this.repository.createEntity(dto);
      if (!data) {
        throw new HttpException(
          Constants.NOT_FOUND,
          HttpStatus.NOT_FOUND
        );
      }
      return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
    } else {
      let id = isExists[0].id;
      const data = await this.repository.updateEntity(dto, id);
      if (!data) {
        throw new HttpException(
          Constants.UPDATE_FAILED,
          HttpStatus.BAD_REQUEST
        );
      }
      return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
    }
  }

  async findAll(): Promise<HeroSectionLeftCmsDefaultEntity[] | Error> {
    const data = await this.repository.findAll();
    if (!data) {
      throw new HttpException(
        Constants.NOT_FOUND,
        HttpStatus.NOT_FOUND
      );
    }
    return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
  }
}