import { EntityRepository, getConnection, Repository } from "typeorm";
import { UpdateLocationPageCmsDto } from "./dto/update-location-page-cms.dto";
import { LocationPageCmsEntity } from "./entities/location-page-cms.entity";

@EntityRepository(LocationPageCmsEntity)
export class LocationPageCmsRepository extends Repository<LocationPageCmsEntity> {

  async createEntity(dto: UpdateLocationPageCmsDto): Promise<LocationPageCmsEntity | Error> {
    try {
      const entity = this.create({
        locationPgeTitle: dto.locationPgeTitle,
        contactIcon: dto.contactIcon,
        locationIcon: dto.locationIcon
      });
      await this.save(entity);
      return entity;
    } catch (e) {
      return e;
    }
  }

  async updateEntity(dto: UpdateLocationPageCmsDto, id: string): Promise<LocationPageCmsEntity | Error> {
    try {
      await this.update(id, {
        locationPgeTitle: dto.locationPgeTitle,
        contactIcon: dto.contactIcon,
        locationIcon: dto.locationIcon
      });
      return await this.findOne(id);
    } catch (e) {
      return e;
    }
  }

  async findAll(): Promise<LocationPageCmsEntity[] | Error> {
    try {
      return await this.find({
        where: {
          isDeleted: false
        }
      });
    } catch (e) {
      return e;
    }
  }

  async deleteEntity(id: string): Promise<boolean | Error> {
    try {
      const updateResult = await getConnection()
        .createQueryBuilder()
        .update(LocationPageCmsEntity)
        .set({ isDeleted: true })
        .where("id = :id", { id: id })
        .execute();
      if (updateResult.affected > 0) {
        return true;
      }
      return false;
    } catch (e) {
      return e;
    }
  }
}