import { Body, Controller, Delete, Get, Param, Patch, Post, UseGuards } from "@nestjs/common";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { Constants } from "src/utils/constants";
import { BlogCategoryService } from "./blog-category.service";
import { CreateBlogCategoryDto } from "./dto/create-blog-category.dto";
import { UpdateBlogCategoryDto } from "./dto/update-blog-category.dto";

@Controller({ path: "blog-category", version: Constants.API_VERSION_1 })
export class BlogCategoryController {
  constructor(private readonly service: BlogCategoryService) {
  }

  @Post()
  @UseGuards(AdminAuthGuard)
  create(@Body() dto: CreateBlogCategoryDto) {
    return this.service.create(dto);
  }

  @Get()
  @UseGuards(AdminAuthGuard)
  findAll() {
    return this.service.findAll();
  }

  @Get(":id")
  @UseGuards(AdminAuthGuard)
  findOne(@Param("id") id: string) {
    return this.service.findOne(id);
  }

  @Patch(":id")
  @UseGuards(AdminAuthGuard)
  update(@Param("id") id: string, @Body() dto: UpdateBlogCategoryDto) {
    return this.service.update(id, dto);
  }

  @Delete(":id")
  @UseGuards(AdminAuthGuard)
  async remove(@Param("id") id: string) {
    return await this.service.remove(id);
  }
}
