import { Body, Controller, Delete, Get, Param, Patch, Post, UseGuards } from "@nestjs/common";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { Constants } from "src/utils/constants";
import { BlogTagService } from "./blog-tag.service";
import { CreateBlogTagDto } from "./dto/create-blog-tag.dto";
import { UpdateBlogTagDto } from "./dto/update-blog-tag.dto";

@Controller({ path: "blog-tag", version: Constants.API_VERSION_1 })
export class BlogTagController {
  constructor(private readonly service: BlogTagService) {
  }

  @Post()
  @UseGuards(AdminAuthGuard)
  create(@Body() dto: CreateBlogTagDto) {
    return this.service.create(dto);
  }

  @Get()
  @UseGuards(AdminAuthGuard)
  findAll() {
    return this.service.findAll();
  }

  @Get(":id")
  @UseGuards(AdminAuthGuard)
  findOne(@Param("id") id: string) {
    return this.service.findOne(id);
  }

  @Patch(":id")
  @UseGuards(AdminAuthGuard)
  update(@Param("id") id: string, @Body() dto: UpdateBlogTagDto) {
    return this.service.update(id, dto);
  }

  @Delete(":id")
  @UseGuards(AdminAuthGuard)
  async remove(@Param("id") id: string) {
    return await this.service.remove(id);
  }
}
