import { Body, Controller, Delete, Get, Param, Patch, Post, UseGuards } from "@nestjs/common";
import { Constants } from "src/utils/constants";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { ContactUsSectionOneDefaultService } from "./contact-us-section-one-default.service";
import { CreateContactUsSectionOneDefaultDto } from "./dto/create-contact-us-section-one-default.dto";
import { UploadedFiles, UseInterceptors } from "@nestjs/common/decorators";
import { FileFieldsInterceptor } from "@nestjs/platform-express";
import { UploadedMulterFileI } from "../SpacesModule/SpacesService";
import { UpdateContactUsSectionOneDefaultDto } from "./dto/update-contact-us-section-one-default.dto";

@Controller({ path: "contact-us-section-one-default", version: Constants.API_VERSION_1 })

export class ContactUsSectionOneDefaultController {
  constructor(private readonly service: ContactUsSectionOneDefaultService) {
  }

  @Post()
  // @UseGuards(AdminAuthGuard)
  @UseInterceptors(FileFieldsInterceptor(
    [
      { name: "icon", maxCount: 1 }
    ]
  ))
  create(
    @Body() dto: CreateContactUsSectionOneDefaultDto,
    @UploadedFiles() files: {
      icon: UploadedMulterFileI
    }
  ) {
    return this.service.create(dto, files);
  }

  @Get()
  @UseGuards(AdminAuthGuard)
  findAll() {
    return this.service.findAll();
  }

  @Get(":id")
  @UseGuards(AdminAuthGuard)
  findOne(@Param("id") id: string) {
    return this.service.findOne(id);
  }

  @Patch("/:id")
  @UseGuards(AdminAuthGuard)
  @UseInterceptors(FileFieldsInterceptor(
    [
      { name: "icon", maxCount: 1 }
    ]
  ))
  update(
    @Param("id") id: string,
    @Body() dto: UpdateContactUsSectionOneDefaultDto,
    @UploadedFiles() files: {
      icon: UploadedMulterFileI
    }) {
    return this.service.update(id, dto, files);
  }

  @Delete(":id")
  @UseGuards(AdminAuthGuard)
  async remove(@Param("id") id: string) {
    return await this.service.remove(id);
  }
}
