import { Module } from "@nestjs/common";
import { ProductServiceService } from "./product-service.service";
import { ProductServiceController } from "./product-service.controller";
import { TypeOrmModule } from "@nestjs/typeorm";
import { ProductServiceRepository } from "./product-service.repository";
import { JwtModule } from "@nestjs/jwt";
import JwtConfigService from "src/core/jwt/jwt-config.service";
import JwtHelper from "src/core/jwt/jwt.helper";
import { UsersRepository } from "src/modules/users/user.repository";
import { DoSpacesService } from "src/modules/SpacesModule/SpacesService/doSpacesService";
import { DoSpacesServicerovider } from "src/modules/SpacesModule/SpacesService";
import { ElasticsearchModule } from "@nestjs/elasticsearch";
import { ProductCategoryRepository } from "../product-category/product-category.repository";
import { RetailerServiceRepository } from "../retailer-service/retailer-service.repository";
import { ProductEffectsRepository } from "../product-effects/product-effects.repository";

@Module({
  imports: [
    JwtModule.registerAsync({
      useClass: JwtConfigService
    }),
    TypeOrmModule.forFeature([UsersRepository, ProductCategoryRepository]),
    TypeOrmModule.forFeature([ProductServiceRepository, RetailerServiceRepository, ProductEffectsRepository]),
    ElasticsearchModule.registerAsync({
      useFactory: async () => ({
        node: process.env.ELASTIC_NODE,
        auth: {
          // username: process.env.ELASTIC_USER,
          // password: process.env.ELASTIC_PASSWORD,
          apiKey: process.env.ELASTIC_API_KEY
        },
        maxRetries: 10,
        requestTimeout: 60000
      })
    })
  ],
  controllers: [ProductServiceController],
  providers: [
    ProductServiceService,
    JwtHelper,
    DoSpacesService,
    DoSpacesServicerovider
  ]
})
export class ProductServiceModule {
}
