import { Injectable } from "@nestjs/common";
import { fetchAllRetailers, fetchOneRetailer, menuType } from "src/helper/dutchie-plus-query";
import { getGraphQLClientWithApiKey } from "src/helper/graphql-client-helper";
import { InjectRepository } from "@nestjs/typeorm";
import { RetailerServiceRepository } from "../custom/retailer-service/retailer-service.repository";

@Injectable()
export class RetailerService {
  constructor(
    @InjectRepository(RetailerServiceRepository)
    private readonly retailerServiceRepo: RetailerServiceRepository
  ) {
  }

  async filterRetailers(data: any, search: string) {
    try {
      let searchList = [];
      for (let i = 0; i < data.length; i++) {
        const name = data[i].name.toLowerCase();
        const address = data[i].address.toLowerCase();
        const res = name.includes(search.toLowerCase()) || address.includes(search.toLowerCase());
        if (res) {
          searchList.push(data[i]);
        }
      }
      return searchList;
    } catch (e) {
      return e;
    }
  }

  async checkRetailerDifference(data: any): Promise<any | Error> {
    try {
      const retailerService = await this.retailerServiceRepo.findByFilterQuery({});

      if (Object.keys(retailerService).length > 0) {
        const existingRetailerIds = retailerService.map(x => x.retailerId);
        const difference = data.filter(obj => !existingRetailerIds.includes(obj.id));
        console.log("Retailer Difference =", difference);

        const dtoObjects = difference.map(obj => ({
          retailerId: obj.id,
          retailerName: obj.name,
          retailerAddress: JSON.stringify(obj.addressObject),
          retailerDeliverySettings: JSON.stringify(obj.deliverySettings),
          hours: JSON.stringify(obj.hours),
          apiKeyName: obj.apiKeyName
        }));

        await this.retailerServiceRepo.createEntities(dtoObjects);
      } else {
        const dtoObjects = data.map(obj => ({
          retailerId: obj.id,
          retailerName: obj.name,
          retailerAddress: JSON.stringify(obj.addressObject),
          retailerDeliverySettings: JSON.stringify(obj.deliverySettings),
          hours: JSON.stringify(obj.hours),
          apiKeyName: obj.apiKeyName
        }));

        await this.retailerServiceRepo.createEntities(dtoObjects);
      }

    } catch (e) {
      console.log(e);
      return e;
    }
  }

  async fetchRetailers(apiKeyName: string) {
    const graphQLClient = getGraphQLClientWithApiKey(apiKeyName);
    const query = fetchAllRetailers();
    const data = await graphQLClient.request(query);
    return data.retailers.map(retailer => ({ ...retailer, apiKeyName }));
  }

  async findAll(type: string, search: string) {
    try {
      const promises = [
        this.fetchRetailers("DUTCHIE_PLUS_API_TOKEN_1"),
        this.fetchRetailers("DUTCHIE_PLUS_API_TOKEN_2")
      ];
      const [retailers1, retailers2] = await Promise.all(promises);
      const mergedRetailers = [...retailers1, ...retailers2];

      // store retailers in local storage
      // await this.checkRetailerDifference(mergedRetailers);

      // check retailer type
      if (type === "DELIVERY") {
        let deliveryList = [];
        for (let i = 0; i < mergedRetailers.length; i++) {
          if (mergedRetailers[i].deliverySettings.afterHoursOrderingForDelivery) {
            const delivery = mergedRetailers[i].hours.delivery;
            if (
              delivery.Sunday.active &&
              delivery.Monday.active &&
              delivery.Tuesday.active &&
              delivery.Wednesday.active &&
              delivery.Thursday.active &&
              delivery.Friday.active &&
              delivery.Saturday.active
            ) {
              deliveryList.push(mergedRetailers[i]);
            }
          }
        }

        if (typeof search !== "undefined") {
          return await this.filterRetailers(deliveryList, search);
        } else {
          return deliveryList;
        }
      } else if (type === "PICKUP") {
        let pickupList = [];
        for (let i = 0; i < mergedRetailers.length; i++) {
          if (mergedRetailers[i].deliverySettings.afterHoursOrderingForPickup) {
            const pickup = mergedRetailers[i].hours.pickup;
            if (
              pickup.Sunday.active &&
              pickup.Monday.active &&
              pickup.Tuesday.active &&
              pickup.Wednesday.active &&
              pickup.Thursday.active &&
              pickup.Friday.active &&
              pickup.Saturday.active
            ) {
              pickupList.push(mergedRetailers[i]);
            }
          }
        }

        if (typeof search !== "undefined") {
          return await this.filterRetailers(pickupList, search);
        } else {
          return pickupList;
        }
      } else {
        if (typeof search !== "undefined") {
          return await this.filterRetailers(mergedRetailers, search);
        } else {
          return mergedRetailers;
        }
      }
    } catch (e) {
      return e;
    }
  }

  async findOne(variables: any) {
    try {
      const retailerService = await this.retailerServiceRepo.findOne({
        where: {
          retailerId: variables.retailerId,
          isDeleted: false
        }
      });
      if (retailerService) {
        const graphQLClient = getGraphQLClientWithApiKey(retailerService.apiKeyName);
        const query = fetchOneRetailer();
        const data = await graphQLClient.request(query, variables);
        if (data) {
          if (retailerService) {
            await this.retailerServiceRepo.update(retailerService.id, {
              accessCount: retailerService.accessCount + 1
            });
          }
          return data;
        }
      }
    } catch (e) {
      return e;
    }
  }

  async menuType(variables: any) {
    try {
      const retailerService = await this.retailerServiceRepo.findOne({
        where: {
          retailerId: variables.retailerId,
          isDeleted: false
        }
      });
      if (retailerService) {
        const graphQLClient = getGraphQLClientWithApiKey(retailerService.apiKeyName);
        const query = menuType();
        const data = await graphQLClient.request(query, variables);
        return data["retailer"];
      } else {
        return [];
      }
    } catch (e) {
      return e;
    }
  }

  async countAllRetailer() {
    try {
      const promises = [
        this.fetchRetailers("DUTCHIE_PLUS_API_TOKEN_1"),
        this.fetchRetailers("DUTCHIE_PLUS_API_TOKEN_2")
      ];
      const [retailers1, retailers2] = await Promise.all(promises);
      const mergedRetailers = [...retailers1, ...retailers2];
      return mergedRetailers.length;
    } catch (e) {
      return e;
    }
  }
}
